<?php
namespace jvbpdelement\Modules\Meta\Widgets;

use Elementor\Repeater;
use Elementor\Icons_Manager;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

class Module_Repeater_Meta extends Base {
	public function get_name() { return 'jvbpd-module-repeater-meta'; }
	public function get_title() { return 'Module Repeater Meta'; }
    public function get_icon() { return 'eicon-image-rollover'; }
    public function get_categories() { return [ 'jvbpd-page-builder-module' ]; }

	protected function _register_controls() {
        $this->start_controls_section( 'section_general', Array(
            'label' => esc_html__("General", 'jvfrmtd'),
        ));
            $this->add_control( 'meta', Array(
                'type' => Controls_Manager::REPEATER,
                'label' => esc_html__("Meta Field", 'jvfrmtd'),
                'fields' => $this->getRepeaterItems(),
            ));
        $this->end_controls_section();
        $this->start_controls_section( 'section_style_font', Array(
            'tab' => Controls_Manager::TAB_STYLE,
            'label' => esc_html__( 'Style', 'elementor' ),
        ));
            $this->add_responsive_control('meta_icon_size', Array(
                'type' => Controls_Manager::SLIDER,
                'label' => esc_html__( 'Size', 'elementor' ),
                'range' => Array(
                    'px' => Array(
                        'min' => 0,
                        'max' => 100,
                    )
                ),
                'selectors' => Array(
                    '{{WRAPPER}} .jvbpd-repeater-meta-wrap' => 'font-size: {{SIZE}}{{UNIT}}',
                )
            ));

            $this->add_responsive_control('meta_color', Array(
                'type' => Controls_Manager::COLOR,
                'label' => esc_html__( 'Meta Color', 'elementor' ),
                'selectors' => Array(
                    '{{WRAPPER}} .jvbpd-repeater-meta-wrap' => 'color: {{VALUE}}',
                )
            ));

            $this->add_responsive_control('meta_icon_color', Array(
                'type' => Controls_Manager::COLOR,
                'label' => esc_html__( 'Icon Color', 'elementor' ),
                'selectors' => Array(
                    '{{WRAPPER}} .jvbpd-repeater-meta-wrap .icon-wrap' => 'color: {{VALUE}}',
                )
            ));

        $this->end_controls_section();
    }

	protected function render() {
        $this->add_render_attribute('wrap', 'class', 'jvbpd-repeater-meta-wrap');
        $output_format = '{%1$s}';
        ?>
        <div <?php echo $this->get_render_attribute_string('wrap'); ?>>
            <?php
            foreach($this->get_settings('meta') as $meta ){
                $this->add_render_attribute('meta', 'class', Array('jvbpd-repeater-meta', 'meta-'.$meta['field']), true);

                if('preview' == $meta['field'] && 'yes' == $meta['hide_preview_label']) {
                    $this->add_render_attribute('meta', 'class', 'hide-preview-label');
                } ?>
                <div <?php echo $this->get_render_attribute_string('meta'); ?>>
                    <?php
                    if('yes' == $meta['use_icon']) {
                        printf('<div class="icon-wrap d-inline-block">');
                        Icons_Manager::render_icon( $meta['icon'], Array(
                            'aria-hidden' => 'true',
                            'style' => 'display:flex;'
                        ) );
                        printf("</div>");
                    } ?>
                    <?php printf( $output_format, $meta['field'] ); ?>
                </div>
                <?php
            } ?>
        </div>
        <?php
    }

    // protected function content_template() { parent::__card_content_template(); }
    public function getRepeaterItems() {
        $repeater = new Repeater();
        $repeater->add_control('field', Array(
            'type' => Controls_Manager::SELECT2,
            'label' => __( 'Button Type', 'jvfrmtd' ),
            'label_block' => true,
            'default' => 'post_title',
            'options' => $this->getReplaceOptions(),
        ));
        $repeater->add_control('hide_preview_label', Array(
            'type' => Controls_Manager::SWITCHER,
            'label' => __( 'Hide Preview Label', 'jvfrmtd' ),
            'default' => 'yes',
            'selectors' => Array(
                '{{WRAPPER}} .hide-preview-label .brief-label' => 'display:none;',
            ),
            'condition' => Array(
                'field' => 'preview',
            ),
        ));
        $repeater->add_control('use_icon', Array(
            'type' => Controls_Manager::SWITCHER,
            'label' => __( 'Use Icon', 'jvfrmtd' ),
        ));
        $repeater->add_control('icon', Array(
            'type' => Controls_Manager::ICONS,
            'show_label' => false,
            'default' => Array(
                'value' => 'fas fa-star',
                'library' => 'fa-solid',
            ),
            'condition' => Array(
                'use_icon' => 'yes',
            ),
        ));
        return $repeater->get_controls();
    }
}