<?php
namespace jvbpdelement\Modules\Userforms\Widgets;

class jv_login extends Base {

	public function get_name() { return parent::LOGIN; }
	public function get_title() { return 'JV Login'; }
	public function get_icon() { return 'eicon-lock-user'; }

	protected function _register_controls() {
		parent::_register_controls();

		if(get_option('users_can_register')) {
			$this->start_injection(Array(
				'at' => 'after',
				'of' => 'show_register',
			));
				$this->add_control('register_link_type', Array(
					'type' => \Elementor\Controls_Manager::SELECT,
					'label' => esc_html__('Register Link Type', 'jvfrmtd'),
					'default' => '',
					'options' => Array(
						'' => esc_html__('Modal', 'jvfrmtd'),
						'link' => esc_html__('Page Link', 'jvfrmtd'),
					),
					'condition' => Array(
						'show_register' => 'yes',
					),
				));
				$this->add_control('register_page_id', Array(
					'type' => \Elementor\Controls_Manager::SELECT,
					'label_block' => true,
					'show_label' => false,
					'default' => '0',
					'options' => Array('0' => esc_html__('Select a page', 'jvfrmtd')) + jvbpd_get_all_pages(),
					'separator' => 'after',
					'condition' => Array(
						'register_link_type' => 'link',
						'show_register' => 'yes',
					),
				));
			$this->end_injection();
		}
	}

	protected function render() {
		parent::render();
		$settings = $this->get_settings();
		$current_url = remove_query_arg( 'fake_arg' );

		$be_redirection = 'yes' === $this->get_settings_for_display('redirect_after_login');
		$this->add_render_attribute( 'form', Array(
			'data-redirect' => $be_redirection ? $this->get_settings_for_display('redirect_url') : null,
		) );

		if ( is_user_logged_in() && ! \Elementor\Plugin::$instance->editor->is_edit_mode() ) {
			if ( 'yes' === $settings['show_logged_in_message'] ) {
				$current_user = wp_get_current_user();
				echo '<div class="elementor-login">' .
					sprintf( __( 'You are Logged in as %1$s (<a href="%2$s">Logout</a>)', 'jvfrmtd' ), $current_user->display_name, wp_logout_url( $current_url ) ) .
					'</div>';
			}else{
				echo '<script type="text/javascript">' .
					sprintf('window.location.href="%1$s";', home_url('/')).
				'</script>';
			}
			return;
		}

	 $field_size ="";
	 if( !empty( $settings['input_size'] ) ) {
		$field_size = "field-size-". $settings['input_size'];
	 }


	$btn_size="";
	if( !empty( $settings['button_size'] ) ) {
		$btn_size = "btn-size-". $settings['button_size'];
	}

	$btn_block="";
	if ($settings['button_block'] == 'yes' ) {
		$btn_block = "btn-block";
	}

	$this->add_render_attribute( 'form', Array(
		'class' => 'jvbpd-login-form jv-modal-login',
		'method' => 'post',
		'action' => esc_url( wp_login_url( apply_filters( 'jvbpd_modal_login_redirect', '' ) ) ),
	) );

	$this->add_render_attribute( 'output', Array(
		'class' => 'btn-wrap submit-wrap',
	) );

	if('link' == $this->get_settings_for_display('register_link_type')) {
		$this->add_render_attribute( 'signup-link', Array(
			'href' => get_permalink($this->get_settings_for_display('register_page_id')),
			'class' => 'required',
		) );
	}else{
		$this->add_render_attribute( 'signup-link', Array(
			'href' => '#',
			'data-toggle' => 'modal',
			'data-target' => '#register_panel',
			'data-dismiss' => 'modal',
			'class' => 'required',
		) );
	} ?>
		<form <?php echo $this->get_render_attribute_string( 'form' ); ?>>
			<div class="row">
				<div class="col-md-12 lava_login_wrap">

					<!-- User Name -->
					<div class="form-group">
						<label class="" for="reg-usernmae"><?php echo $this->get_settings_for_display('user_label'); ?></label>
						<input type="text" name="log" id="username"  value="" class="form-control <?php echo $field_size ?>" placeholder="<?php echo $this->get_settings_for_display('user_placeholder'); ?>" required>
					</div>

					<!-- User Password -->
					<div class="form-group">
						<label class="" for="reg-password"><?php echo $this->get_settings_for_display('password_label'); ?></label>
						<input type="password" name="pwd" id="password" value="" class="form-control <?php echo $field_size ?>" placeholder="<?php echo $this->get_settings_for_display('password_placeholder'); ?>" required>
					</div>

					<!-- Descriptions -->
					<div class="form-group">
						<?php if( $settings['show_remember_me'] ) : ?>
						<label class="control-label">
							<input name="rememberme" value="forever" type="checkbox"> <?php esc_html_e("Remember Me", 'jvfrmtd' );?>
						</label><!-- /.control-label -->
						<?php endif; ?>
						<?php if( $settings['show_lost_password'] ) : ?>
						<a href="<?php echo esc_url( wp_lostpassword_url() );?>">
							<p class="required"><?php esc_html_e('Forgot Your Password?', 'jvfrmtd' ); ?></p>
						</a>
						<?php endif; ?>
					</div>

					<?php
					'yes' == $this->get_settings_for_display('use_nocaptcha') &&
					self::isActiveNoCaptcha() && self::render_noCaptcha(); ?>


					<!-- Login Button -->
					<div class="form-group">
						<div class="lava_login">
							<div <?php echo $this->get_render_attribute_string( 'output' ); ?>>
								<div class="button-wrap">
									<button type="submit" class="btn <?php echo $btn_size; ?> <?php echo $btn_block; ?>"><?php echo $settings['button_text']; ?></button>
								</div>
								<?php do_action( 'jvbpd_login2_modal_login_after' ); ?>
							</div><!-- /.col-md-12 -->
						</div><!-- /.row -->
					</div>
				</div>
			</div><!--/.row-->

			<div class="row bottom_row">
				<hr class="padding-5px">
				<div class="col-md-12 col-xs-12">
					<?php if( get_option( 'users_can_register' ) ) : ?>
						<?php if( $settings['show_register'] ) : ?>
							<p><?php esc_html_e("Don't have an account?", 'jvfrmtd' );?> <a <?php echo $this->get_render_attribute_string( 'signup-link' ); ?>><?php esc_html_e('Sign Up', 'jvfrmtd' );?></a></p>
						<?php endif; ?>
					<?php else: ?>
						<div class="not_allowed_new_member">
							<span><?php esc_html_e("This site is not allowed new members. please contact administrator.", 'jvfrmtd' );?></span>
						</div>
					<?php endif; ?>
				</div>
			</div>
			<input type="hidden" name="security" value="<?php echo wp_create_nonce( 'user_login' ); ?>">
			<input type="hidden" name="referer" value="<?php echo isset( $_GET[ 'referer' ] ) && '' != $_GET[ 'referer' ] ? esc_url( $_GET[ 'referer' ] ) : false; ?>">
		</form>
		<?php
	}
}