<?php
/**
 * Widget Name: Header Widget
 * Author: Javo
 * Version: 1.0.0.0
*/

namespace jvbpdelement\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Icons_Manager;
use Elementor\Repeater;

if( !defined( 'ABSPATH' ) ) {
    exit;
}

class Jvbpd_Map_Panels extends Widget_Base {

    public function get_name() { return 'jvbpd-map-panels'; }
	public function get_title() { return 'Map Panels'; }
    public function get_icon() { return 'eicon-button'; }
    public function get_categories() { return [ 'jvbpd-elements' ]; }

	protected function _register_controls() {
        $this->start_controls_section( 'section_general', array(
			'label' => esc_html__( 'General', 'jvfrmtd' ),
        ) );
            $this->add_control( 'panel', Array(
                'label' => __( 'Panels', 'jvfrmtd' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $this->getPanelOptions(),
                'title_field' => '{{{ elementor.helpers.renderIcon( this, icon, {}, "i", "panel" ) || ' .
                '\'<i class="{{ icon }}" aria-hidden="true"></i>\' }}}' .
                '{{{ text }}} ({{{ type }}})',
            ) );
        $this->end_controls_section();
    }

    public function getPanelOptions() {
        $repeater = new Repeater();
        $repeater->add_control('type', Array(
            'type' => Controls_Manager::SELECT,
            'label' => esc_html__( 'Type', 'jvfrmtd' ),
            'label_block' => true,
            'default' => 'logo',
            'seprator' => 'after',
            'options' => Array(
                'logo' => esc_html__("Logo", 'jvfrmtd'),
                'all_list' => esc_html__("All List", 'jvfrmtd'),
                'add_listing' => esc_html__("Add Listing", 'jvfrmtd'),
                'custom_link' => esc_html__("Custom Link", 'jvfrmtd'),
                'favorites' => esc_html__("Favorites", 'jvfrmtd'),
                'menu' => esc_html__("Menu", 'jvfrmtd'),
            ),
        ));

        // Logo
        $repeater->add_control('logo_image', Array(
            'label' => esc_html__('Icon', 'jvfrmtd'),
            'type' => Controls_Manager::MEDIA,
            'condition' => Array('type' => 'logo'),
        ));

        // Favorites
        $repeater->add_control( 'favorites_des', Array(
            'type' => Controls_Manager::RAW_HTML,
            'raw'  => '<div class="elementor-jv-notice" style="background-color:#9b0a46; color:#ffc6c6; padding:10px;">'.
                esc_html__('Note : The "Favorites" panel is available only when the "Enable Detail Panel" option is used in the "Listing Blocks (List Type)" widget','jvfrmtd') .
            '</div>',
            'condition' => Array('type' => 'favorites'),
        ));

        // without Logo
        $repeater->add_control('icon', Array(
            'type' => Controls_Manager::ICONS,
            'label' => esc_html__('Icon', 'jvfrmtd'),
            'default' => Array(
                'value' => 'fas fa-plus',
                'library' => 'fa-solid',
            ),
            'condition' => Array('type!' => 'logo'),
        ));

        $repeater->add_control('text', Array(
            'type' => Controls_Manager::TEXT,
            'label' => esc_html__('Text', 'jvfrmtd'),
            'default' => esc_html__('Label', 'jvfrmtd'),
            'condition' => Array('type!' => 'logo'),
        ));

        $repeater->add_control( 'submit_page_id', Array(
            'type' => Controls_Manager::SELECT2,
            'label' => esc_html__( 'Add Listing Page', 'jvfrmtd' ),
            'label_block' => true,
            'default' => '0',
            'seprator' => 'before',
            'options' => jvbpd_get_all_pages(),
            'condition' => Array('type' => 'add_listing'),
        ) );

        // Custom link
        $repeater->add_control( 'custom_link', Array(
            'type' => Controls_Manager::URL,
            'label' => esc_html__( 'URL', 'jvfrmtd' ),
            'label_block' => true,
            'seprator' => 'before',
            'condition' => Array('type' => 'custom_link'),
        ) );

        // Menu
        $repeater->add_control( 'menu_id', Array(
            'type' => Controls_Manager::SELECT2,
            'label' => esc_html__( 'Menu Template', 'jvfrmtd' ),
            'label_block' => true,
            'seprator' => 'before',
            'options' => jvbpd_elements_tools()->getCanvasIDs(),
            'condition' => Array('type' => 'menu'),
            'description' => esc_html__('Please select a canvas menu template. If you don`t have any, please create one in Javo page builder ( Javo setting > Page Builder ).', 'jvfrmtd'),
        ) );

        return $repeater->get_controls();
    }

    protected function render() {
        $panels = $this->get_settings_for_display('panel');
        $this->add_render_attribute( 'panel_list', 'class', 'jvbpd-map-panels-list-items' );
        ?>
        <ul <?php echo $this->get_render_attribute_string( 'panel_list' ); ?>>
            <?php foreach($panels as $panel) { ?>

                <?php
                $this->add_render_attribute( 'panel_list_item', 'class', 'jvbpd-map-panels-list-item', true);
                $this->add_render_attribute( 'panel_list_item', Array(
                    'class' => 'type-' . $panel['type'],
                ));

                $this->add_render_attribute( 'panel_list_item_link', Array(
                    'class' => Array('jvbpd-map-panels-list-items-link', 'type-' . $panel['type'], 'd-inline-block'),
                ), null, true);

                $toLink = 'javascript:void(0);';
                $toTarget = '_self';
                $toRel = '';
                $menuID = '';
                switch($panel['type']) {
                case 'logo':
                    $toLink = home_url('/');
                    break;
                case 'add_listing':
                    $submitPageID = isset($panel['submit_page_id']) ? $panel['submit_page_id'] : 0;
                    if(\WP_Post::get_instance($submitPageID)) {
                        $toLink = get_permalink($submitPageID);
                        $toTarget = '_blank';
                    }
                    break;
                case 'menu':
                    $menu_id = isset($panel['menu_id']) ? $panel['menu_id'] : 0;
                    if(\WP_Post::get_instance($menu_id)) {
                        $menuID = $menu_id;
                    }
                    break;
                case 'custom_link':
                    $toLink = $panel['custom_link']['url'];
                    $toTarget = !empty($panel['custom_link']['is_external']) ? '_blank' : $toTarget;
                    $toRel = !empty($panel['custom_link']['nofollow']) ? 'nofollow' : $toRel;
                    break;
                }

                $this->add_render_attribute('panel_list_item_link', Array(
                    'href' => $toLink,
                    'target' => $toTarget,
                    'rel' => $toRel,
                    'data-menu-id' => $menuID,
                ), null, true); ?>

                <li <?php echo $this->get_render_attribute_string( 'panel_list_item' ); ?>>
                    <a <?php echo $this->get_render_attribute_string( 'panel_list_item_link' ); ?>>
                        <?php if('logo' == $panel['type']) { ?>
                            <img src="<?php echo esc_attr($panel['logo_image']['url']); ?>">
                        <?php }else{ ?>
                            <div class="jvbpd-map-panel-icon">
                                <?php Icons_Manager::render_icon($panel['icon'], Array('aria-hidden' => 'true')); ?>
                            </div>
                            <div class="jvbpd-map-panel-label"><?php echo esc_html($panel['text']);?></div>
                        <?php } ?>
                    </a>
                </li>
            <?php } ?>
        </ul>
        <?php

    }
}