<?php
/**
 * Widget Name: Map Review Filter
 * Author: Javo
 * Version: 1.0.0.1
*/

namespace jvbpdelement\Widgets;

use Elementor\Repeater;
use Elementor\Widget_Base;
use Elementor\Icons_Manager;
use Elementor\Controls_Manager;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

class Jvbpd_Map_Review_Filter extends Widget_Base {

	public function get_name() { return 'jvbpd-map-review-filter'; }
	public function get_title() { return 'Map Review Filter'; }
	public function get_icon() { return 'fa fa-map-o'; }
    public function get_categories() { return [ 'jvbpd-map-page' ]; }

    public function getFilterOptions() {

        $filters = lv_directoryReview()->core->get_option( 'use_map_filter', Array() );

        $repeater = new Repeater();
        $repeater->add_control('filter', Array(
            'type' => Controls_Manager::SELECT,
            'label' => esc_html__( 'Filter', 'jvfrmtd' ),
            'label_block' => true,
            'default' => '',
            'seprator' => 'after',
            'options' => Array(
                '' => esc_html__( 'None', 'jvfrmtd' )
            ) + array_combine($filters, $filters),
        ));

        return $repeater->get_controls();
    }

    protected function _register_controls() {
        $this->start_controls_section('section_general', Array(
            'label' => esc_html__( 'General', 'jvfrmtd' ),
        ));
            $this->add_control( 'filters', Array(
                'label' => __( 'Filters', 'jvfrmtd' ),
                'type' => Controls_Manager::REPEATER,
                'fields' => $this->getFilterOptions(),
                'title_field' => '{{{ filter }}}',
            ) );
        $this->end_controls_section();
    }

    protected function render() {
        $this->add_render_attribute('filter-button-wrap', 'class', Array(
            'jvbpd-review-filter-wrap', 'd-inline-block', 'dropdown'
        ));
        $filters = $this->get_settings_for_display('filters');
        foreach($filters as $filter) {
            if('' == $filter['filter']) {
                continue;
            }
            $this->add_render_attribute('filter-button', Array(
                'class' => Array('btn', 'btn-primary', 'btn-sm', 'dropdown-toggle'),
                'data-toggle' => 'dropdown',
                'data-review-filter' => $filter['filter'],
            ), false, true);
            ?>
            <div <?php echo $this->get_render_attribute_string('filter-button-wrap'); ?>>
                <button <?php echo $this->get_render_attribute_string('filter-button'); ?>>
                    <?php esc_html_e($filter['filter'], 'jvfrmtd'); ?>
                </button>
                <div class="dropdown-menu" aria-labelledby="dropdownMenuButton">
                    <a href="javascript:void(0);" class="dropdown-item" data-score="">
                        <?php esc_html_e("Unset", 'jvfrmtd'); ?>
                    </a>
                    <?php
                    for($buttonLoop=0; $buttonLoop < 5; $buttonLoop++) {
                        printf(
                            '<a href="javascript:void(0);" class="dropdown-item" data-score="%1$s">%1$s</a>', ($buttonLoop+1)
                        );
                    } ?>
                </div>
            </div>
            <?php
        }
    }
}