<?php
namespace jvbpdelement\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Icons_Manager;
use Elementor\Group_Control_Typography;
use Elementor\Scheme_Typography;
use Elementor\Scheme_Color;
use Elementor\Utils;
use Elementor\Slider;
use Elementor\Group_Control_Background;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Elementor tabs widget.
 *
 * Elementor widget that displays vertical or horizontal tabs with different
 * pieces of content.
 *
 * @since 1.0.0
 */
class jvbpd_tabs extends Widget_Base {

	public function get_name() { return 'jv_tabs'; }
	public function get_title() { return __( 'JV Tabs', 'jvfrmtd' ); }
	public function get_icon() { return 'eicon-tabs'; }
	public function get_categories() { return [ 'jvbpd-elements' ]; }

	protected function _register_controls() {
		$this->start_controls_section( 'section_tabs',[
				'label' => __( 'JV Tabs', 'jvfrmtd' ),
		]);
		$this->add_control( 'hide_label', Array(
			'label' => __( 'Hide Label & Use Tooltip', 'jvfrmtd' ),
			'type' => Controls_Manager::SWITCHER,
			'default' => '',
		));

		$this->add_control(
			'tab_style',
			[
				'label' => __( 'Style', 'jvfrmtd' ),
				'type' => Controls_Manager::SELECT,
				'default' => '',
				'options' => [
					'' => __( 'Default', 'jvfrmtd' ),
					'img-right' => __( 'Image Right', 'jvfrmtd' ),
					'img-full' => __( 'Image Full', 'jvfrmtd' ),
					'img-full-width' => __( 'Image Full Width', 'jvfrmtd' ),
				],
			]
		);

		$this->add_responsive_control('tab_bg_padding',
		[
			'label'         => esc_html__('Tab BG Padding', 'jvfrmtd'),
			'type'          => Controls_Manager::DIMENSIONS,
			'size_units'    => ['px'],
			'selectors'     => [
				'{{WRAPPER}} .jv-nav-wrap-bg' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
			],
			'condition' => [
				'tab_style' => 'img-full-width'
			],
			'separator' => 'after',
	        ]
		);

		$this->add_control( 'tab_wrap_bg_color', Array(
			'label' => __( 'Tab Background Color', 'jvfrmtd' ),
			'type' => Controls_Manager::COLOR,
			'selectors' => Array(
				'{{WRAPPER}} .jv-nav-wrap-bg' => 'background-color: {{VALUE}};',
			),
			'condition' => [
				'tab_style' => 'img-full-width'
			],
		));

		$this->add_responsive_control(
			'tab_wrap_width',
			[
				'label' => __( 'Tab Wrap Width', 'jvfrmtd' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ '%', 'px' ],
				'range' => [
					'%' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
					'px' => [
						'min' => 0,
						'max' => 1800,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 1140,
				],
				'selectors' => [
					'{{WRAPPER}} .img-full-width' => 'width: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
                    'tab_style' => 'img-full-width'
				],
			]
		);

		/** Image Full **/
		$this->add_responsive_control(
			'content_inner_left',
			[
				'label' => __( 'Content Inner Left', 'jvfrmtd' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 1,
					]
				],
				'selectors' => [
					'{{WRAPPER}} .jv-nav-wrap.img-full .tab_content' => 'margin-left: {{SIZE}}{{UNIT}};',
				],
				'separator' => 'before',
				'condition' => [
                    'tab_style' => 'img-full'
				],
			]
		);

		$this->add_responsive_control(
			'content_inner_left_width',
			[
				'label' => __( 'Content Inner Width', 'jvfrmtd' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ '%', 'px' ],
				'range' => [
					'%' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
					'px' => [
						'min' => 0,
						'max' => 1800,
						'step' => 1,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .jv-nav-wrap.img-full .tab_content' => 'width: {{SIZE}}{{UNIT}};',
				],
				'condition' => [
                    'tab_style' => 'img-full'
				],
			]
		);


		$this->add_responsive_control('content_header_padding',
		[
			'label'         => esc_html__('Tab Header Padding (Position)', 'jvfrmtd'),
			'type'          => Controls_Manager::DIMENSIONS,
			'size_units'    => ['px'],
			'selectors'     => [
				'{{WRAPPER}} .jv-nav-wrap.img-full .nav' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
			],
			'condition' => [
				'tab_style' => 'img-full'
			],
			'separator' => 'after',
	        ]
		);

		/** Img Right **/
		$this->add_responsive_control(
			'content_inner_imgright_left_width',
			[
				'label' => __( 'Content Inner Width', 'jvfrmtd' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ '%', 'px' ],
				'range' => [
					'%' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
					'px' => [
						'min' => 0,
						'max' => 1800,
						'step' => 1,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .jv-nav-wrap.img-right .tab_content' => 'flex: 0 1 auto; max-width:{{SIZE}}{{UNIT}};',
				],
				'condition' => [
					'tab_style' => 'img-right'
				],
			]
		);

		$this->add_control(
			'horizontal_alignment',
			[
				'label' => __( 'Alignment', 'jvfrmtd' ),
				'type' => Controls_Manager::SELECT,
				'default' => '',
				'options' => [
					'' => __( 'Default', 'jvfrmtd' ),
					'nav-justified' => __( 'Justified', 'jvfrmtd' ),
					'nav-fill' => __( 'Fill', 'jvfrmtd' ),
					'justify-content-center' => __( 'Justify content center', 'jvfrmtd' ),
					'justify-content-end' => __( 'Justify content end', 'jvfrmtd' ),
					//'flex-column' => __( 'flex-column', 'jvfrmtd' ),  // It needs to add wrap div like col-md-3, col-md-9
				],
			]
		);


		$this->add_control(
			'tabs_pill',
			[
				'label' => __( 'Tabs or Pills', 'jvfrmtd' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'nav-pills',
				'options' => [
					'nav-tabs' => __( 'Tabs', 'jvfrmtd' ),
					'nav-pills' => __( 'Pills', 'jvfrmtd' ),
					'nav-under-line' => __( 'Under Line', 'jvfrmtd' ),
				],
			]
		);

		$this->add_control(
			'tabs',
			[
				'label' => __( 'Tabs Items', 'jvfrmtd' ),
				'type' => Controls_Manager::REPEATER,
				'default' => [
					[
						'tab_title' => __( 'Tab #1', 'jvfrmtd' ),
						'tab_content' => __( 'I am tab content. Click edit button to change this text. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.', 'jvfrmtd' ),
					],
					[
						'tab_title' => __( 'Tab #2', 'jvfrmtd' ),
						'tab_content' => __( 'I am tab content. Click edit button to change this text. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.', 'jvfrmtd' ),
					],
				],
				'fields' => [

					[
						'name' => 'tab_icon',
						'label' => __( 'Tab Icon', 'jvfrmtd' ),
						'type' => Controls_Manager::ICONS,
						'default' => Array(
							'value' => '',
							'library' => '',
						),
					],

					[
						'name' => 'tab_title',
						'label' => __( 'Tab Title', 'jvfrmtd' ),
						'type' => Controls_Manager::TEXT,
						'default' => __( 'Tab Title', 'jvfrmtd' ),
						'placeholder' => __( 'Tab Title', 'jvfrmtd' ),
						'label_block' => true,
					],

					[
						'name' => 'tab_content_title',
						'label' => __( 'Content Title', 'jvfrmtd' ),
						'type' => Controls_Manager::TEXT,
						'placeholder' => __( 'Tab Title', 'jvfrmtd' ),
						'label_block' => true,
					],

					[
						'name' => 'tab_content',
						'label' => __( 'Content', 'jvfrmtd' ),
						'default' => __( 'Tab Content', 'jvfrmtd' ),
						'placeholder' => __( 'Tab Content', 'jvfrmtd' ),
						'type' => Controls_Manager::WYSIWYG,
						'show_label' => false,
					],

					[
						'name' => 'tab_content_bg_switcher',
						'label' => __( 'Background color or image', 'jvfrmtd' ),
						'type' => Controls_Manager::SWITCHER,
						'label_on' => __( 'Color', 'jvfrmtd' ),
						'label_off' => __( 'IMG', 'jvfrmtd' ),
						'return_value' => 'yes',
						'default' => 'yes',
					],


					[
						'name' => 'tab_content_bgcolor',
						'label' => __( 'Background Color', 'jvfrmtd' ),
						'type' => Controls_Manager::COLOR,
						'default' => '#fbe183',
						'scheme' => [
							'type' => Scheme_Color::get_type(),
							'value' => Scheme_Color::COLOR_4,
						],
						'condition' => [
							'tab_content_bg_switcher' => 'yes'
						],
					],

					[
						'name' => 'tab_content_bgimg',
						'label' => __( 'Background Image', 'jvfrmtd' ),
						'type' => Controls_Manager::MEDIA,
						'default' => [
							'url' => \Elementor\Utils::get_placeholder_image_src(),
						],
						'condition' => [
							'tab_content_bg_switcher!' => 'yes'
						],
					],


					[
						'name' => 'tab_content_img_switcher',
						'label' => __( 'Add an Image', 'jvfrmtd' ),
						'type' => Controls_Manager::SWITCHER,
						'label_on' => __( 'yes', 'jvfrmtd' ),
						'label_off' => __( 'no', 'jvfrmtd' ),
						'return_value' => 'yes',
						'default' => '',
					],

					[
						'name' => 'tab_content_img',
						'label' => __( 'Image', 'jvfrmtd' ),
						'type' => Controls_Manager::MEDIA,
						'default' => [
							'url' => \Elementor\Utils::get_placeholder_image_src(),
						],
						'condition' => [
							'tab_content_img_switcher' => 'yes'
						],
					],

					[
						'name' => 'tab_content_img_size',
						'label' => __( 'Image size', 'jvfrmtd' ),
						'type' => Controls_Manager::SLIDER,
						'size_units' => [ 'px', '%' ],
						'range' => [
							'px' => [
								'min' => 0,
								'max' => 1000,
								'step' => 1,
							],
							'%' => [
								'min' => 0,
								'max' => 100,
							],
						],
						'default' => [
							'unit' => '%',
							'size' => 100,
						],
						'selectors' => [
							'{{WRAPPER}} {{CURRENT_ITEM}} .tab_content_img img' => 'width: {{SIZE}}{{UNIT}};',
						],
						'condition' => [
							'tab_content_img_switcher' => 'yes'
						],
					],

					[
						'name' => 'tab_content_img_position_x',
						'label' => __( 'Position top', 'jvfrmtd' ),
						'type' => Controls_Manager::SLIDER,
						'size_units' => [ 'px', '%' ],
						'range' => [
							'px' => [
								'min' => 0,
								'max' => 1000,
								'step' => 1,
							],
							'%' => [
								'min' => 0,
								'max' => 100,
							],
						],
						'default' => [
							'unit' => '%',
							'size' => 35,
						],
						'selectors' => [
							'{{WRAPPER}} {{CURRENT_ITEM}} .tab_content_img' => 'top: {{SIZE}}{{UNIT}};',
						],
						'condition' => [
							'tab_content_img_switcher' => 'yes'
						],
					],

					[
						'name' => 'tab_content_img_position_y',
						'label' => __( 'Position left', 'jvfrmtd' ),
						'type' => Controls_Manager::SLIDER,
						'size_units' => [ 'px', '%' ],
						'range' => [
							'px' => [
								'min' => 0,
								'max' => 2000,
								'step' => 1,
							],
							'%' => [
								'min' => 0,
								'max' => 100,
							],
						],
						'default' => [
							'unit' => '%',
							'size' => 50,
						],
						'selectors' => [
							'{{WRAPPER}} {{CURRENT_ITEM}} .tab_content_img' => 'left: {{SIZE}}{{UNIT}};',
						],
						'condition' => [
							'tab_content_img_switcher' => 'yes'
						],
					],

					[
						'name' => 'tab_content_btn_txt',
						'label' => __( 'Button Text', 'jvfrmtd' ),
						'type' => Controls_Manager::TEXT,
						'placeholder' => __( 'Button Text', 'jvfrmtd' ),
						'label_block' => true,
					],

					[
						'name' => 'tab_content_btn_url',
						'label' => __( 'Button URL', 'jvfrmtd' ),
						'type' => Controls_Manager::URL,
						'placeholder' => __( 'https://your-link.com', 'jvfrmtd' ),
						'show_external' => true,
						'default' => [
							'url' => '',
							'is_external' => true,
							'nofollow' => true,
						],
					],

				],

				'title_field' => '{{{ tab_title }}}',
			]
		);



		$this->end_controls_section();

		$this->start_controls_section(
			'section_tabs_style',
			[
				'label' => __( 'Tabs Title', 'jvfrmtd' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'tabs_arrage',
			[
				'label' => __( 'Tabs Arrage', 'jvfrmtd' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'default' => 'horizon',
				'options' => [
					'horizon'  => __( 'Horizone', 'jvfrmtd' ),
					'vertical' => __( 'Vertical', 'jvfrmtd' ),
				],
			]
		);

        $this->add_responsive_control('tab_padding',
		 [
			'label'         => esc_html__('Padding', 'jvfrmtd'),
			'type'          => Controls_Manager::DIMENSIONS,
			'size_units'    => ['px', 'em', '%'],
			'default' => Array(
				'top' => 10,
				'right' => 20,
				'bottom' => 10,
				'left' => 20,
				'unit' => 'px'
			),
			'selectors'     => [
				'{{WRAPPER}} .nav-tabs .nav-link, {{WRAPPER}} .nav-pills .nav-link' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				]
			]
         );


		$this->add_control( 'tab_background_color', Array(
			'label' => __( 'Tab Header Background Color', 'jvfrmtd' ),
			'type' => Controls_Manager::COLOR,
			'selectors' => Array(
				'{{WRAPPER}} .nav-link' => 'background-color: {{VALUE}};',
				'{{WRAPPER}} .nav-link' => 'background-color: {{VALUE}};',
			),
		));
		$this->add_control( 'tab_active_background_color', Array(
			'label' => __( 'Selected Tab Background Color', 'jvfrmtd' ),
			'type' => Controls_Manager::COLOR,
			'selectors' => Array(
				'{{WRAPPER}} .nav-link.active' => 'background-color: {{VALUE}};',
				'{{WRAPPER}} .nav-link.active' => 'background-color: {{VALUE}};',
			),
		));

		$this->add_control(
			'heading_title',
			[
				'label' => __( 'Title', 'jvfrmtd' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'tab_color',
			[
				'label' => __( 'Color', 'jvfrmtd' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .nav-link' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' => Scheme_Color::get_type(),
					'value' => Scheme_Color::COLOR_1,
				],
			]
		);

		$this->add_control(
			'tab_active_color',
			[
				'label' => __( 'Active Color', 'jvfrmtd' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .nav-link.active' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' => Scheme_Color::get_type(),
					'value' => Scheme_Color::COLOR_4,
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'tab_typography',
				'selector' => '{{WRAPPER}} .nav-link',
				'scheme' => Scheme_Typography::TYPOGRAPHY_1,
			]
		);

		$this->add_control(
			'tab_icon',
			[
				'label' => __( 'Tab Icon (default : title typho)', 'jvfrmtd' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'tab_icon_color',
			[
				'label' => __( 'Color', 'jvfrmtd' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .nav-link i' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' => Scheme_Color::get_type(),
					'value' => Scheme_Color::COLOR_1,
				],
			]
		);

		$this->add_control(
			'tab_icon_active_color',
			[
				'label' => __( 'Active Icon Color', 'jvfrmtd' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .nav-link.active i' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' => Scheme_Color::get_type(),
					'value' => Scheme_Color::COLOR_4,
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'tab_icon_typography',
				'selector' => '{{WRAPPER}} .nav-link i',
				'scheme' => Scheme_Typography::TYPOGRAPHY_1,
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'section_contents_style',
			[
				'label' => __( 'Tabs Contents', 'jvfrmtd' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);


        $this->add_responsive_control('tab_content_padding',
		 [
			'label'         => esc_html__('Wrap Padding', 'jvfrmtd'),
			'type'          => Controls_Manager::DIMENSIONS,
			'size_units'    => ['px', 'em', '%'],
			'selectors'     => [
				'{{WRAPPER}} .tab-content'=> 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				]
			]
		 );

		$this->add_responsive_control('tab_content_inner_padding',
		 [
			'label'         => esc_html__('Inner Padding', 'jvfrmtd'),
			'type'          => Controls_Manager::DIMENSIONS,
			'size_units'    => ['px', 'em', '%'],
			'default' => Array(
				'top' => 20,
				'right' => 20,
				'bottom' => 20,
				'left' => 20,
				'unit' => 'px'
			),
			'selectors'     => [
				'{{WRAPPER}} .tab_content'=> 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				]
			]
		 );

		 $this->add_control(
			'content_title',
			[
				'label' => __( 'Content Title', 'jvfrmtd' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'content_title_color',
			[
				'label' => __( 'Content title Color', 'jvfrmtd' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} h3.tab_content_title' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' => Scheme_Color::get_type(),
					'value' => Scheme_Color::COLOR_3,
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'content_title_typography',
				'selector' => '{{WRAPPER}} h3.tab_content_title',
				'scheme' => Scheme_Typography::TYPOGRAPHY_3,
			]
		);

		$this->add_control(
			'heading_content',
			[
				'label' => __( 'Content', 'jvfrmtd' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'content_color',
			[
				'label' => __( 'Content Color', 'jvfrmtd' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tab_content_des' => 'color: {{VALUE}};',
				],
				'scheme' => [
					'type' => Scheme_Color::get_type(),
					'value' => Scheme_Color::COLOR_3,
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'content_typography',
				'selector' => '{{WRAPPER}} .tab_content_des',
				'scheme' => Scheme_Typography::TYPOGRAPHY_3,
			]
		);

		$this->add_responsive_control('content_content_padding',
		[
			'label'         => esc_html__('Content Padding', 'jvfrmtd'),
			'type'          => Controls_Manager::DIMENSIONS,
			'size_units'    => ['px'],
			'selectors'     => [
				'{{WRAPPER}} .tab_content_des' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
			],
			'separator' => 'after',
	        ]
		);


		$this->add_control(
			'tabcontent_min_height',
			[
				'label' => __( 'Tab Content Min Height', 'jvfrmtd' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 1000,
						'step' => 5,
					],
					'%' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 50,
				],
				'selectors' => [
					'{{WRAPPER}} .tab-content>.tab-pane' => 'min-height: {{SIZE}}{{UNIT}};',
				],
				'separator' => 'before',
			]
		);
		$this->end_controls_section();
	}

	/**
	 * Render tabs widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function render() {
		$tabs = $this->get_settings_for_display( 'tabs' );
		$settings = $this->get_settings();

		$id_int = substr( $this->get_id_int(), 0, 3 );

		$this->add_render_attribute( 'nav_style_class', [
			'class' => 'nav',
		] );

		$this->add_render_attribute( 'nav_wrap_bg', [
			'class' => 'jv-nav-wrap-bg',
		] );

		$this->add_render_attribute( 'nav_wrap', [
			'class' => 'jv-nav-wrap',
		] );


		if( !empty($settings['tab_style']) ) {
			$this->add_render_attribute( 'nav_wrap', [
				'class' => $settings['tab_style'],
			] );
		}

		if( !empty($settings['horizontal_alignment']) ) {
			$this->add_render_attribute( 'nav_style_class', [
				'class' => $settings['horizontal_alignment'],
			] );
		}

		if( !empty($settings['tabs_arrage']) ) {
			$this->add_render_attribute( 'nav_style_class', [
				'class' => $settings['tabs_arrage'],
			] );
		}

		if( !empty($settings['tabs_pill']) ) {
			$this->add_render_attribute( 'nav_style_class', [
				'class' => $settings['tabs_pill'],
			] );
		} ?>
		<?php if(in_array($this->get_settings_for_display('tab_style'), Array('img-full-width'))) { ?>
		<div <?php echo $this->get_render_attribute_string( 'nav_wrap_bg' ); ?>>
		<?php } ?>
			<div <?php echo $this->get_render_attribute_string( 'nav_wrap' ); ?>>
				<nav>
					<div <?php echo $this->get_render_attribute_string( 'nav_style_class' ); ?> id="nav-tab" role="tablist">
					<?php
					$tabs_post_id = '';
					if(isset($GLOBALS['javo_map_ajax_loop_post_id'])) {
						$tabs_post_id = $GLOBALS['javo_map_ajax_loop_post_id'] . '-';
					}
					foreach ( $tabs as $index => $item ) :
						$tab_count = $index + 1;

						$tab_title_setting_key = $this->get_repeater_setting_key( 'tab_title', 'tabs', $index );

						$this->add_render_attribute( $tab_title_setting_key, [
							'class' => [ 'nav-item', 'nav-link' ],
							'id' => 'nav-tab-title-' . $tabs_post_id . $id_int . $tab_count,
							'data-toggle' => 'tab',
							'href' => '#nav-' . $tabs_post_id . $id_int . $tab_count,
							'role' => 'tab',
							'data-tab' => $tab_count,
							'data-bg-type' => 'yes' == $item['tab_content_bg_switcher'] ? 'color' : 'image',
							'data-bg-color' => $item['tab_content_bgcolor'],
							'data-bg-src' => $item['tab_content_bgimg']['url'],
							'aria-controls' => 'nav-' . $id_int . $tab_count,
							//'aria-selected' => 'false',
						], false, true );

						if('yes' == $this->get_settings_for_display('hide_label')) {
							$this->add_render_attribute( $tab_title_setting_key, Array(
								'class' => 'javo-tooltip',
								'title' => $item['tab_title'],
							) );
						}

						if( $tab_count == 1 ) {
							$this->add_render_attribute( $tab_title_setting_key, [
								'class' => 'active',
							] );
						} ?>
						<a <?php echo $this->get_render_attribute_string( $tab_title_setting_key ); ?>>
							<?php
							Icons_Manager::render_icon( $item['tab_icon'], Array(
								'aria-hidden' => 'true',
							) );
							if('yes' != $this->get_settings_for_display('hide_label')){
								printf(' ');
								printf('<span class="tab-title">%1$s</span>', $item['tab_title']);
							} ?>
						</a>
					<?php endforeach; ?>
					<div class="nav_underline"></div>
					</div>
				</nav>

				<div class="tab-content" id="nav-tabContent">
					<?php
					foreach ( $tabs as $index => $item ) :
						$tab_count = $index + 1;

						$tab_content_setting_key = $this->get_repeater_setting_key( 'tab_content', 'tabs', $index );
						$this->add_render_attribute( $tab_content_setting_key, [
							'class' => [ 'tab-pane', 'fade', 'elementor-repeater-item-' . $item['_id'] ],
							'id' => 'nav-' . $tabs_post_id . $id_int . $tab_count,
							'role' => 'tabpanel',
							'aria-labelledby' => 'nav-' . $id_int . $tab_count,
							//'style' => 'background-image: url("'. $tab_content_background . '");',
						] );

						if( $tab_count == 1 ) {
							$this->add_render_attribute( $tab_content_setting_key, [
								'class' => ['active', 'show'],
							] );
						}
						$target = $item['tab_content_btn_url']['is_external'] ? ' target="_blank"' : '';
						$nofollow = $item['tab_content_btn_url']['nofollow'] ? ' rel="nofollow"' : '';
						$btn = '<a href="' . $item['tab_content_btn_url']['url'] . '"' . $target . $nofollow . ' class="tab_content_btn">'. $item['tab_content_btn_txt'] .' </a>';
						?>
						<div <?php echo $this->get_render_attribute_string( $tab_content_setting_key ); ?> >
							<div class="tab_content">
								<?php if( !empty($item['tab_content_title']) ) { ?>
									<h3 class="tab_content_title"><?php echo $item['tab_content_title']; ?></h3>
								<?php } ?>
								<div class="tab_content_des"><?php echo $this->parse_text_editor( $item['tab_content'] ); ?></div>
								<?php if( !empty($item['tab_content_btn_txt']) ) { ?>
									<?php echo $btn; ?>
								<?php } ?>
							</div>

							<?php if( $item['tab_content_img_switcher'] == 'yes' ) { ?>
								<div class="tab_content_img">
									<img src="<?php echo $item['tab_content_img']['url']; ?>"/>
								</div>
							<?php } ?>

							<?php if(in_array($this->get_settings_for_display('tab_style'), Array('img-full', 'img-right'))) { ?>
								<div class="tab_content_bgimg">
									<img src="<?php echo $item['tab_content_bgimg']['url']; ?>"/>
								</div>
							<?php } ?>

						</div>
					<?php endforeach; ?>
				</div>

			</div>
		<?php if(in_array($this->get_settings_for_display('tab_style'), Array('img-full-width'))) { ?>
		</div>
		<?php } ?>

		<?php
	}

	/**
	 * Render tabs widget output in the editor.
	 *
	 * Written as a Backbone JavaScript template and used to generate the live preview.
	 *
	 * @since 1.0.0
	 * @access protected
	 */

}
