<?php
/**
 * Settings API
 *
 *
 */

namespace Javo\Api;

/**
 * Settings API Class
 */
class Settings
{
	const PURCHASECODE_KEY = '_javo_purchase_code';
	const PURCHASECODE_VERIFY = '_javo_purchase_verify';

	const DEV_KEY = 'javo-dev';
	public $remote_url = 'https://wpjavo.com/theme/wp-json/javo_activate/';

	/**
	 * Settings array
	 * @var private array
	 */
	public $settings = array();

	/**
	 * Sections array
	 * @var private array
	 */
	public $sections = array();

	/**
	 * Fields array
	 * @var private array
	 */
	public $fields = array();

	/**
	 * Script path
	 * @var string
	 */
	public $script_path;

	/**
	 * Enqueues array
	 * @var private array
	 */
	public $enqueues = array();

	/**
	 * Admin pages array to enqueue scripts
	 * @var private array
	 */
	public $enqueue_on_pages = array();

	/**
	 * Admin pages array
	 * @var private array
	 */
	public $admin_pages = array();

	/**
	 * Admin subpages array
	 * @var private array
	 */
	public $admin_subpages = array();

	private static $activateMessages = Array();

	/**
	 * Constructor
	 */
	public function __construct()
	{}

	public function register() {
		self::requestPurchaseCode();

		if(!self::isActiveTheme()) {
			add_action('admin_notices', Array($this, 'themeActivateNotice'));
			// add_action('admin_menu', Array($this, 'themeActivatePage'));
		}else{
			// add_action('wp_loaded', Array($this, 'activateRedirect'));
		}

		if(!function_exists('jvbpdCore')) {
			add_action('admin_menu', Array($this, 'themeActivatePage'));
		}

		$this->defines();
		$this->loadWalker();
		$this->loadPlugins();
		$this->loadTSettings();
	}

	/**
	 * Dinamically enqueue styles and scripts in admin area
	 *
	 * @param  array  $scripts file paths or wp related keywords of embedded files
	 * @param  array  $pages    pages id where to load scripts
	 */
	public function admin_enqueue( $scripts = array(), $pages = array() )
	{
		if ( empty( $scripts ) )
			return;

		$i = 0;
		foreach ( $scripts as $key => $value ) :
			foreach ( $value as $val ):
				$this->enqueues[ $i ] = $this->enqueue_script( $val, $key );
				$i++;
			endforeach;
		endforeach;

		if ( !empty( $pages ) ) :
			$this->enqueue_on_pages = $pages;
		endif;

		return $this;
	}

	/**
	 * Call the right WP functions based on the file or string passed
	 *
	 * @param  array $script  file path or wp related keyword of embedded file
	 * @param  var $type      style | script
	 * @return variable functions
	 */
	private function enqueue_script( $script, $type ) {
		if ( $script === 'media_uplaoder' )
			return 'wp_enqueue_media';

		return ( $type === 'style' ) ? array( 'wp_enqueue_style' => $script ) : array( 'wp_enqueue_script' => $script );
	}

	/**
	 * Injects user's defined pages array into $admin_pages array
	 *
	 * @param  var $pages      array of user's defined pages
	 */
	public function addPages( $pages )
	{
		$this->admin_pages = $pages;

		return $this;
	}

	public function withSubPage( $title = null )
	{
		if ( empty( $this->admin_pages ) ) {
			return $this;
		}

		$adminPage = $this->admin_pages[0];

		$subpage = array(
			array(
				'parent_slug' => $adminPage['menu_slug'],
				'page_title' => $adminPage['page_title'],
				'menu_title' => ($title) ? $title : $adminPage['menu_title'],
				'capability' => $adminPage['capability'],
				'menu_slug' => $adminPage['menu_slug'],
				'callback' => $adminPage['callback']
			)
		);

		$this->admin_subpages = $subpage;

		return $this;
	}

	/**
	 * Injects user's defined pages array into $admin_subpages array
	 *
	 * @param  var $pages      array of user's defined pages
	 */
	public function addSubPages( $pages )
	{
		$this->admin_subpages = ( count( $this->admin_subpages ) == 0 ) ? $pages : array_merge( $this->admin_subpages, $pages );

		return $this;
	}

	/**
	 * Injects user's defined settings array into $settings array
	 *
	 * @param  var $args      array of user's defined settings
	 */
	public function add_settings( $args )
	{
		$this->settings = $args;

		return $this;
	}

	/**
	 * Injects user's defined sections array into $sections array
	 *
	 * @param  var $args      array of user's defined sections
	 */
	public function add_sections( $args )
	{
		$this->sections = $args;

		return $this;
	}

	/**
	 * Injects user's defined fields array into $fields array
	 *
	 * @param  var $args      array of user's defined fields
	 */
	public function add_fields( $args )
	{
		$this->fields = $args;

		return $this;
	}


	public function defines() {
		$defines = Array(
			'JVBPD_APP_PATH' => get_template_directory(),
			'JVBPD_THEME_DIR' => get_template_directory_uri(),
			'JVBPD_IMG_DIR' => get_template_directory_uri() . '/assets/dist/images/',
		);
		foreach( $defines as $defineKey => $defineValue ) {
			if( !defined($defineKey) ) {
				define( $defineKey, $defineValue );
			}
		}
	}

	public function loadWalker() {
		$walkerPath = get_template_directory() . '/inc/Api/Callbacks/walker-callback.php';

		if( file_exists( $walkerPath ) ) {
		 	require_once( $walkerPath );
		}
	}

	public function loadTSettings() {
		$settingsPath = get_template_directory() . '/inc/Api/t-settings.php';
		if( file_exists( $settingsPath ) ) {
			require_once( $settingsPath );
		}
	}

	public function loadPlugins() {
		$pluginsPath = get_template_directory() . '/inc/Api/Plugins.php';
		if( file_exists( $pluginsPath ) ) {
			require_once( $pluginsPath );
		}
		$tgmpaPath = get_template_directory() . '/inc/Custom/tgmpa.php';
		if(file_exists($tgmpaPath)) {
			require_once($tgmpaPath);
		}
	}

	public function api_call( $endpoint, $body_args = [], $force = false ) {
        $response = wp_remote_post($this->remote_url . $endpoint, Array(
            'user-agent' => 'Mozilla/5.0 ' . home_url(),
            'timeout'    => 10,
            'body'       => $body_args,
        ));

        if ( is_wp_error( $response ) ) {
            return $response;
        }

		$data = json_decode( wp_remote_retrieve_body( $response ), true );

        if ( empty( $data ) || ! is_array( $data ) ) {
            return new \WP_Error( 'no_json', '', [
                __( 'An error occurred, please try again', 'jvbpd' ),
                var_export( wp_remote_retrieve_body( $response ), true )
            ] );
        }
        return $data;
	}

	public static function requestPurchaseCode() {
		if(isset($_POST['javo_theme_activate'])) {
			$Settings = new self;
			$invalidCodeError = new \WP_Error('invaild-code', esc_html__("The purchase code is invalid. please try again", 'jvbpd'));
			$javoPurchaseCode = isset($_POST['purchase_code']) ? $_POST['purchase_code'] : '';

			if(!empty($javoPurchaseCode)) {
				if(self::DEV_KEY == $javoPurchaseCode){
					$result = Array('status' => 'success');
				}else{
					$result = $Settings->api_call('v1/verify', Array(
						'code' => $javoPurchaseCode,
						'domain' => home_url('/'),
						'name' => wp_get_theme()->get('Name'),
						'version' => wp_get_theme()->get('Version'),
					));
				}
			}else{
				$result = $invalidCodeError;
			}

			if(is_wp_error($result)) {
				self::$activateMessages[] = $result->get_error_message();
			}else{
				if(isset($result['status']) && 'success' == $result['status']) {
					self::setPurchaseCode($javoPurchaseCode);
				}else{
					self::$activateMessages[] = isset($result['message']) ? $result['message'] : $invalidCodeError->get_error_message();
				}
			}
		}
		if(isset($_POST['javo_theme_deactivate'])) {
			self::setPurchaseCode();
		}
	}

	public static function getActivateMessage() {
		if(0 < sizeof(self::$activateMessages)) {
			?>
			<ul>
				<?php
				foreach(self::$activateMessages as $message) {
					printf('<li>%1$s</li>', $message);
				} ?>
			</ul>
			<?php
		}
	}

	public static function getActivateForm() {
		?>
		<form method="post" id="jvbpd-verify-purchasecode-form">
			<label>
				<?php esc_html_e("Purchase code : ", 'jvbpd'); ?>
				<input type="text" name="purchase_code" value="" class="regular-text">
			</label>

			<?php submit_button(esc_html__("Activate", 'jvbpd')); ?>
			<input type="hidden" name="javo_theme_activate">
			<p>
				<a href="<?php echo esc_url('https://help.market.envato.com/hc/en-us/articles/202822600-Where-Is-My-Purchase-Code-'); ?>" target="_blank"><?php esc_html_e("Where Is My Purchase Code?", 'jvbpd'); ?></a>
			</p>
		</form>
		<?php
	}

	public static function getDeactivateForm() {
		?>
		<form method="post" id="jvbpd-remove-purchasecode-form">
			<?php submit_button(esc_html__("Deactivate", 'jvbpd')); ?>
			<input type="hidden" name="javo_theme_deactivate">
		</form>
		<?php
	}

	public function themeActivatePage() {
		add_menu_page(
			esc_html__("Javo Theme Activate", 'jvbpd'),
			'Javo Activate',
			'manage_options',
			'jvbpd_admin',
			Array($this, 'render_jvbpd_activate'), '', 3
		);
		if(isset($_GET['page']) && 'jvbpd_admin' == $_GET['page']) {
			add_action('admin_enqueue_scripts', Array(__CLASS__, 'admin_enqueues'));
		}
	}

	public function activateRedirect() {
		if(isset($_GET['page']) && 'jvbpd_activate' == $_GET['page']) {
			wp_safe_redirect(admin_url('admin.php?page=jvbpd_admin_activate'));
			exit;
		}
	}

	public static function admin_enqueues() {
		wp_enqueue_style( 'javo-settings-style', get_template_directory_uri() . '/assets/dist/css/admin.css' );
	}

	public function themeActivateNotice() {
		if('toplevel_page_jvbpd_admin' == get_current_screen()->id) {
			return;
		} ?>
		<div class="notice notice-success is-dismissible">
			<h1><?php esc_html_e( 'Javo Themes', 'jvbpd' ); ?></h1>
			<p><?php esc_html_e( 'Please activate your license to get premium support, and unlimited access to the template library and demo importer.', 'jvbpd' ); ?></p>
			<p>
				<a href="<?php echo admin_url('admin.php?page=jvbpd_admin'); ?>" class="button button-primary"><?php esc_html_e("Activate License", 'jvbpd'); ?></a>
			</p>
		</div>
		<?php
	}

	public static function getPurchaseCode() { return get_option(self::PURCHASECODE_KEY); }
	public static function setPurchaseCode($key=false) {
		if($key) {
			update_option(self::PURCHASECODE_VERIFY, 'verified');
			update_option(self::PURCHASECODE_KEY, $key);
		}else{
			delete_option(self::PURCHASECODE_VERIFY);
			delete_option(self::PURCHASECODE_KEY);
		}
	}

	public static function isActiveTheme() {
		return 'verified';
		return 'verified' == get_option(self::PURCHASECODE_VERIFY);
	}

	public function render_jvbpd_activate() {
		get_template_part('views/admin-helper/activate');
	}
}